function g = kempgradient(X, Y, ps)

% KEMPGRADIENT Compute gradient of data-point likelihood wrt x.
beta = ps.beta;
sigma = exp(X(1)); 
X = X(2:end);
if sigma == 0 || sigma >1e10 || sigma < 1e-20
  g = 0*X; return;
end

numData = size(Y,1);
Xdim  = 2;
origX = reshape(X', numData, Xdim);

dLdX = 0*X; dLdsigma=0;

if ps.missingdata
  for c = 1:ps.runps.chunknum
    obsind = ps.runps.objind{c};
    Xind = [obsind'; obsind'+numData];
    featind = ps.runps.featind{c};
    newY = Y(obsind,featind); newX = origX(obsind,:); newX = newX(:)';
    [o1 o2] = gradient_noprior(newX,newY,sigma,ps);
    dLdX(Xind) = dLdX(Xind) + o1;
    dLdsigma = dLdsigma+o2;
  end
else
  [dLdX dLdsigma] = gradient_noprior(X,Y,sigma,ps);
end

% account for prior
dLdX = dLdX - X;

dLdsigmaprior = -2/sigma + 1/(beta*sigma^2) + 1/sigma;
				    % this because we want derivative with
				    % respect to log(sigma)
dLdsigma = (dLdsigma+dLdsigmaprior)*sigma;

g = [-dLdsigma,-dLdX(:)'];


function [dLdX dLdsigma] = gradient_noprior(X,Y,sigma,ps)

if strcmp(ps.runps.type, 'feat')
  dataDim = size(Y, 2);
  YY = Y*Y';
else
  dataDim = ps.simdim;
  YY = Y*dataDim;
end

numData = size(Y, 1);
Xdim = 2;
X = reshape(X', numData,Xdim);

[Kold, invKold]= kempcomputekernel(X, sigma, ps);

dLdK = 0.5*(invKold*YY*invKold - dataDim*invKold);
epsilon = ps.eps;

dLdX = zeros(numData, Xdim);
for n = 1:numData
  for j = 1:Xdim
    Xndiff = repmat(X(n,:), numData,1) - X;
    dvec = -Kold(:,n).*(X(n,j) - X(:,j))./...
			((sigma)*(sum(Xndiff.*Xndiff, 2) + epsilon).^0.5);
    dLdX(n,j) = 2*dLdK(n,:)*dvec; % - dLdK(n,n)*dvec(n); this is zero
  end
end

% NB: .* important
dKdsigma = -1/sigma*log(2*pi*Kold).*Kold; 

dLdsigma = sum(sum(dLdK.*dKdsigma));
dLdX = dLdX(:)';


