function L = kemplikelihood(X, Y, ps)

beta = ps.beta;
% KEMPLIKELIHOOD Calculate the likelihood of the data set.

Xorig = X;
sigma = exp(X(1)); 
if sigma == 0 || sigma >1e10 || sigma < 1e-20
  L = inf; return;
end
X = X(2:end);

numData = size(Y,1);
Xdim  = 2;
origX = reshape(X', numData, Xdim);
L = 0;

if ps.missingdata
  for c = 1:ps.runps.chunknum
    obsind = ps.runps.objind{c};
    featind = ps.runps.featind{c};
    newY = Y(obsind,featind); newX = origX(obsind,:); newX = newX(:)';
    L = L+likelihood_noprior(newX, newY, sigma, ps);
  end
else
  L = likelihood_noprior(X,Y,sigma,ps);
end


% Include priors over parameters for MAP estimate
L = L - .5*sum(sum(X.*X));

% include sigma prior
L = L  -log(beta) - 2*log(sigma) - 1/(beta*sigma) + log(sigma);

% because we'll be minimizing this
L = -L;

function L = likelihood_noprior(X,Y,sigma,ps)

if strcmp(ps.runps.type, 'feat')
  dataDim = size(Y, 2);
  SS = Y*Y';
elseif strcmp(ps.runps.type, 'sim')
  dataDim = ps.simdim;
  SS = dataDim*Y;
else
  error('unexpected type');
end

numData = size(Y, 1);
X = reshape(X', numData,2);

[K, invK] = kempcomputekernel(X, sigma, ps);
  
L = -dataDim*numData/2*log(2*pi)-dataDim/2*logdet(K);
L = L - 0.5*trace(SS*invK);

